#pragma once

#include <string>
#include <vector>
#include <memory>
#include <database/export.h>

namespace pdp {

struct DatabaseParameters;
class SqlQuery;

/**
 * Takes the string list provided and constructs a string for use in the 
 * SQL "IN" construct.  The result of passing in a 
 * string list = first, second, third will result in a string with the
 * value = "('first', 'second', 'third')"
 *
 * @ingroup database
 */
PDPDATABASE_EXPORT std::string createSqlList(
  const std::vector<std::string>& elements);

/**
 * This function will return a string list with the ID's in the query.  It is
 * assumed that the first column is the id field.
 *
 * @ingroup database
 */
PDPDATABASE_EXPORT std::vector<std::string> extractColumnFromQuery(
  const SqlQuery* query, 
  const int column);

/**
 * Convenience function to convert a string list into a std::vector of
 * ints. Since this function intends to process row ids, only the unique ids are
 * returned in the resultant vector.
 *
 * @ingroup database
 */
PDPDATABASE_EXPORT std::vector<int> convertIdStringListToVector(
  const std::vector<std::string>& list);

/**
 * Constructs the connect string for postgres.  The same as pgConnectString
 * except that the user is set for "postgres"
 *
 * @ingroup database
 */
PDPDATABASE_EXPORT std::string pgConnectStringPostgresUser(
  const std::string& ip,
  const int port,
  const std::string& database,
  bool ssl);

PDPDATABASE_EXPORT std::string pgUnprivilegedUsername();

/**
 * Constructs the connect string for postgres.  
 *
 * @ingroup database
 */
PDPDATABASE_EXPORT std::string pgConnectString(
  const std::string& user,
  const std::string& ip,
  const int port,
  const std::string& database,
  bool ssl);

} //namespace pdp

