/** @file
 *
 * Tello commands socket
 *
 * @author Paul Pantalone
 * @copyright Paul Pantalone. All Rights Reserved
 * @date 2019
 */
#pragma once

#include <functional>
#include <boost/asio.hpp>
#include <utilities/UdpSendingSocket.h>
#include <models/export.h>

using boost::asio::ip::udp;

namespace pdp {

//=====================================================================================
// Command socket
//=====================================================================================
class PDPMODELS_EXPORT TelloCommandSocket : public UdpSendingSocket
{
public:
  TelloCommandSocket(
    const std::string& droneIp,
    unsigned short dronePort,
    unsigned short commandPort,
    std::function<void(const std::string&, uint8_t, const std::string&)>* sendCommandResponseFunction);

  virtual ~TelloCommandSocket() {}

  virtual void sendCommandResponse(
    const std::string& command,
    uint8_t rc,
    const std::string& str) override;

private:
  std::function<void(const std::string&, uint8_t, const std::string&)>* m_sendCommandResponseFunction;
};

} // namespace pdp
