/** @file
 *
 * Tello manager
 *
 * @author Paul Pantalone
 * @copyright Paul Pantalone. All Rights Reserved
 * @date 2019
 */
#pragma once

#include <boost/asio.hpp>
#include <models/Robot.h>
#include <models/TelloCommandSocket.h>
#include <models/TelloStateSocket.h>
#include <models/TelloVideoSocket.h>

using boost::asio::ip::udp;

namespace pdp {

//=====================================================================================
// Tello driver implements Tello SDK 1.3 and 2.0
//=====================================================================================
class TelloManager: public Robot
{
public:
  explicit TelloManager(
    std::function<void()>* commandResponseFunction,
    std::function<void(std::string)>* stateUpdateFunction,
    std::function<void(const unsigned char*, int, int)>* frameDisplayFunction,
    std::function<void()>* commandsTimeoutFunction,
    std::function<void()>* stateTimeoutFunction,
    std::function<void()>* videoTimeoutFunction)
    : Robot(commandResponseFunction, stateUpdateFunction, frameDisplayFunction,
        commandsTimeoutFunction, stateTimeoutFunction, videoTimeoutFunction),
      m_commandSocket(nullptr),
      m_stateSocket(nullptr),
      m_videoSocket(nullptr),
      m_slow(false)
  { m_sendCommandResponseFunction = std::bind(&TelloManager::sendCommandResponse, this,
    std::placeholders::_1, std::placeholders::_2, std::placeholders::_3); }

  ~TelloManager()
  { disconnect(); }

  virtual bool connect(
    unsigned int pollingIntervalInMilliSeconds) override;

  virtual std::string log() override { return std::string(); }

  virtual std::string parameters() override { return std::string(); }

  virtual void launch() override;

  virtual void setSlow() override;

  virtual void land() override;

  virtual void stop() override;

  virtual void hover() override;

  virtual void setVelocities(
    float leftRight,
    float backwardForward,
    float downUp,
    float ccwYawCcYaw) override;

  virtual void disconnect() override;

private:
  virtual void ping() override;

  void sendCommandResponse(
    const std::string& command,
    uint8_t rc,
    const std::string& str);

private:
  TelloCommandSocket* m_commandSocket;
  TelloStateSocket* m_stateSocket;
  TelloVideoSocket* m_videoSocket;
  bool m_slow;
  std::function<void(const std::string&, uint8_t, const std::string&)> m_sendCommandResponseFunction;
};

} // namespace pdp
