/** @file
 *
 * Tello video socket
 *
 * @author Paul Pantalone
 * @copyright Paul Pantalone. All Rights Reserved
 * @date 2018
 */
#pragma once

#include <boost/asio.hpp>
#include <utilities/UdpListeningSocket.h>
#include <imaging/H264decoder.h>
#include <models/export.h>

using boost::asio::ip::udp;

namespace pdp {

class PDPMODELS_EXPORT TelloVideoSocket : public UdpListeningSocket
{
public:
  TelloVideoSocket(
    unsigned short videoPort,
    std::function<void(const unsigned char*, int, int)>* displayFrameFunction);

  virtual ~TelloVideoSocket();

private:
  void processPacket(
    size_t r) override;

  void decodeFrames();

private:
  std::vector<unsigned char> m_seqBuffer;   // Collect video packets into a larger sequence
  size_t m_seqBufferNext;                   // Next available spot in the sequence buffer
  int m_seqBufferNumPackets;                // How many packets we've collected, for debugging
  pdp::H264Decoder m_decoder;               // Decodes h264
  pdp::ConverterRGB24 m_converter;          // Converts pixels from YUV420P to RGB24
  unsigned char* m_frameDisplayBuffer;
  std::function<void(const unsigned char*, int, int)>* m_displayFrameFunction;
};

} // namespace pdp
