#pragma once

#include <QMouseEvent>
#include <QLabel>
#include <QTreeWidget>
#include <utilities/stringUtils.h>
#include <elements/Color.h>
#include <imaging/Image2dDetectionsAdaptor.h>
#include <qtElements/PropertiesDialog.h>
#include <qtElements/export.h>

/**
 * Column label dialog.
 */
class ColumnLabelDlg : public PropertiesDialog
{
Q_OBJECT

public:
  ColumnLabelDlg(
    QWidget *parent);

  ~ColumnLabelDlg();

  void setLabelText(
    std::string labelText)
  { m_label->setText(labelText.c_str()); }

  /**
   * Makes the dialog visible
   */
  void setDialogVisible(
    int globalX,
    int globalY);

private:
  void setupGUI();

private:
  QFont m_panelFont;
  QLabel* m_label;
  int m_width;
  int m_height;
};

class DatasetsTreeWidgetItem : public QTreeWidgetItem
{
public:
  DatasetsTreeWidgetItem(
    QTreeWidget* parent)
    : QTreeWidgetItem(parent)
  {}

private:
  bool operator<(const QTreeWidgetItem &other) const
  {
    int column = treeWidget()->sortColumn();
    if (column == 0)
      return text(column).toInt() < other.text(column).toInt();
    else if (column > 0 && column < 5)
      return text(column) < other.text(column);
    else
    {
      std::vector<std::string> stringList1 = pdp::split(text(column).toStdString(), ",");
      std::vector<std::string> stringList2 = pdp::split(other.text(column).toStdString(), ",");
      int value1 = (stringList1[0] == "") ? -1 : stoi(stringList1[0]);
      int value2 = (stringList2[0] == "") ? -1 : stoi(stringList2[0]);
      return value1 < value2;
    }
  }
};

class DatasetsTableWidget : public QTreeWidget
{
  Q_OBJECT

public:
  DatasetsTableWidget(
    QWidget* parent);

  ~DatasetsTableWidget();

  void setColumns(
    pdp::Image2dDetectionsAdaptor::ClassificationType classificationType,
    const std::vector<std::string>& classLabels,
    const std::vector<pdp::Color>& classColors);

  QTreeWidgetItem* createTableItem(
    int itemIdx,
    std::vector<int>& truthsCount,
    std::vector<int>& truthsDontCaresCount,
    std::vector<int>& classificationsCount,
    bool classified);

  QTreeWidgetItem* updateTableItem(
    int itemIdx,
    std::vector<int>& truthsCount,
    std::vector<int>& truthsDontCaresCount,
    std::vector<int>& classificationsCount,
    bool classified);

  int maxColumns() const
  { return m_maxColumns; }

Q_SIGNALS:
  void sndLoadDataset(
    int idx);

protected:
  virtual void mousePressEvent(
    QMouseEvent* event);

  virtual void mouseDoubleClickEvent(
    QMouseEvent* event);

private:
  void showColumnLabelDialog(
    int columnIdx,
    QPointF localPos,
    QPointF globalPos);

  void setColumnHeader();

  void setColumnLayout();

private:
  struct TableColumn
  {
    int idx;
    std::string label;
  };

  pdp::Image2dDetectionsAdaptor::ClassificationType m_classificationType;
  std::vector<std::string> m_classLabels;
  std::vector<pdp::Color> m_classColors;
  const int m_maxColumns = 12;
  std::vector<TableColumn> m_classColumnIdx;
  ColumnLabelDlg* m_columnLabelDlg;
};
