/** @file
 *
 * Robot rendering widget
 *
 * @author Paul Pantalone
 * @copyright Paul Pantalone. All Rights Reserved
 * @date 2018
 */
#pragma once

#include <QPixmap>
#include <QWidget>
#include <QMouseEvent>
#include <elements/VectorR.h>
#include <models/RobotFactory.h>
#include <render/FlightControlRenderingEngine.h>
#include <qtElements/export.h>

namespace pdp {
class ModelR3f;
}

class PDPQTELEMENTS_EXPORT FlightControlWidget : public QWidget
{
Q_OBJECT

public:
  FlightControlWidget(
    pdp::FlightControlRenderingEngine* engine,
    pdp::ModelR3f* roomModel,
    pdp::ModelR3f* crazyFlieModel,
    pdp::ModelR3f* telloModel,
    QWidget* parent);

  ~FlightControlWidget();

  void render(
    pdp::RobotFactory::RobotType robotType,
    float roll,
    float pitch,
    float yaw);

protected:
  virtual void paintEvent(
    QPaintEvent* paintEvent) override;

private:
  QPixmap m_pixmap;
  pdp::FlightControlRenderingEngine* m_engine;
  pdp::ModelR3f* m_roomModel;
  pdp::ModelR3f* m_crazyFlieModel;
  pdp::ModelR3f* m_telloModel;
};
