#pragma once

#include <future>
#include <utilities/export.h>

namespace pdp {

/**
 * Class that encapsulates promise and future object and
 * provides API to set exit signal for the thread
 *
 * @ingroup utilities
 */
class PDPUTILITIES_EXPORT StoppableTask
{
public:
  StoppableTask()
  : m_futureObj(m_exitSignal.get_future())
  {}
  
  StoppableTask(StoppableTask && obj)
  : m_exitSignal(std::move(obj.m_exitSignal)), m_futureObj(std::move(obj.m_futureObj))
  {}
 
  StoppableTask & operator=(StoppableTask && obj)
  {
    m_exitSignal = std::move(obj.m_exitSignal);
    m_futureObj = std::move(obj.m_futureObj);
    return *this;
  }
 
  // Task need to provide defination  for this function
  // It will be called by thread function
  virtual void run() = 0;
 
  // Thread function to be executed by thread
  void operator()()
  { run(); }
 
  // Checks if thread is requested to stop
  bool stopRequested()
  {
    // checks if value in future object is available
    if (m_futureObj.wait_for(std::chrono::milliseconds(0)) == std::future_status::timeout)
      return false;
    return true;
  }

  // Request the thread to stop by setting value in promise object
  void stop()
  { m_exitSignal.set_value(); }

private:
  std::promise<void> m_exitSignal;
  std::future<void> m_futureObj;
};

} // namespace pdp

