/** @file
 *
 * UDP listening socket base class
 *
 * @author Paul Pantalone
 * @copyright Paul Pantalone. All Rights Reserved
 * @date 2019
 */
#pragma once

#include <boost/asio.hpp>
#include <utilities/export.h>

using boost::asio::ip::udp;

namespace pdp {

class PDPUTILITIES_EXPORT UdpListeningSocket
{
public:
  UdpListeningSocket(
    unsigned short port,
    size_t bufferSize);

  virtual ~UdpListeningSocket() {}

  bool receiving();

  std::chrono::system_clock::time_point receiveTime();

  virtual void timeout();

protected:
  void stopListening();
  
  virtual void processPacket(
    size_t r) = 0;

private:
  // Checks if thread is requested to stop
  bool stopRequested()
  {
    // checks if value in future object is available
    if (m_futureObj.wait_for(std::chrono::milliseconds(0)) == std::future_status::timeout)
      return false;
    return true;
  }

  // Request the thread to stop by setting value in promise object
  void stop()
  { m_exitSignal.set_value(); }

protected:
  const std::vector<unsigned char>& buffer()
  { return m_buffer; }

protected:
  std::mutex m_mtx;                                     // All public calls must be guarded
  bool m_receiving;                                     // Are we receiving packets on this socket?
  std::vector<unsigned char> m_buffer;                  // Packet buffer
  boost::asio::io_service m_ioService;
  udp::socket m_socket;                                 // The socket
  std::chrono::system_clock::time_point m_receiveTime;  // Time of most recent receive
  
private:
  std::thread* m_listeningThread;                       // Listening thread
  std::promise<void> m_exitSignal;
  std::future<void> m_futureObj;
};

} // namespace pdp
