/** @file
 *
 * UDP sending socket base class
 *
 * @author Paul Pantalone
 * @copyright Paul Pantalone. All Rights Reserved
 * @date 2019
 */
#pragma once

#include <utilities/UdpListeningSocket.h>

using boost::asio::ip::udp;

namespace pdp {

class PDPUTILITIES_EXPORT UdpSendingSocket : public UdpListeningSocket
{
public:
  UdpSendingSocket(
    const std::string& targetIp,
    unsigned short targetPort,
    unsigned short port);

  virtual ~UdpSendingSocket()
  { stopListening(); }

  virtual void sendCommandResponse(
    const std::string& command,
    uint8_t rc,
    const std::string& str) = 0;

  void timeout() override;

  bool waiting();

  std::chrono::system_clock::time_point sendTime();

  void sendByteArray(
    unsigned char* byteArray,
    size_t byteArraySize,
    bool commandResponse);

  void sendString(
    const std::string& command,
    bool commandResponse);

protected:
  virtual void processPacket(
    size_t r) override;

private:
  void completeCommand(
    uint8_t rc,
    const std::string& str);

private:
  udp::endpoint m_remoteEndpoint;
  std::chrono::system_clock::time_point m_sendTime;  // Time of most recent send
  bool m_respond;
  std::string m_lastSendString;
  bool m_waiting;
};

} // namespace pdp
