/** @file
 *
 * Property treer utility classes and functions
 *
 * @author Paul Pantalone
 * @copyright Paul Pantalone. All Rights Reserved
 * @date 2018
 */
#pragma once

#include <boost/property_tree/ptree.hpp>
#include <boost/property_tree/json_parser.hpp>
#include <boost/property_tree/xml_parser.hpp>
#include <elements/VectorR.h>
#include <utilities/export.h>

namespace pdp
{

/**
 * Prints the contents of a boost property tree.
 *
 * @ingroup utilities
 */
PDPUTILITIES_EXPORT void printPropertyTree(
  const boost::property_tree::ptree& pt,
  int level,
  std::string& result);

PDPUTILITIES_EXPORT bool parseStringTest(
  const boost::property_tree::ptree& pt,
  const std::string& label,
  std::string& message);

PDPUTILITIES_EXPORT bool parseString(
  const boost::property_tree::ptree& pt,
  const std::string& label,
  std::string& value,
  std::string& message);

PDPUTILITIES_EXPORT bool parseStringOptions(
  const boost::property_tree::ptree& pt,
  const std::string& label,
  const std::vector<std::string>& options,
  std::string& value,
  std::string& message);

PDPUTILITIES_EXPORT bool parseBool(
  const boost::property_tree::ptree& pt,
  const std::string& label,
  bool& value,
  std::string& message);

PDPUTILITIES_EXPORT bool parseInt(
  const boost::property_tree::ptree& pt,
  const std::string& label,
  int& value,
  std::string& message);

PDPUTILITIES_EXPORT bool parseFloat(
  const boost::property_tree::ptree& pt,
  const std::string& label,
  float& value,
  std::string& message);

PDPUTILITIES_EXPORT bool parseVectorR2i(
  const boost::property_tree::ptree& pt,
  const std::string& label,
  VectorR2i& value,
  std::string& message);

} // namespace pdp
