/** @file
 * 
 * USB utility classes and functions for USB devices like cameras and game controllers
 * 
 * @author Paul Pantalone
 * @copyright Paul Pantalone. All Rights Reserved
 * @date 2018
 */
#pragma once

#include <string>
#include <libusb-1.0/libusb.h>
#include <libuvc/libuvc.h>
#include <utilities/export.h>

namespace pdp
{

PDPUTILITIES_EXPORT std::string usbDevicesInfo(
  libusb_device* dev);

/**
 * Inputs a string in the UTC time format of 'yyyy-MM-ddTHH:mm:ss.mmmZ' and returns a time object
 * set to the time value of the input string
 * 
 * @param timeStr Time string with the following time format: 'yyyy-MM-ddTHH:mm:ss.mmmZ'
 * @returns Time object set to the time value of the input string
 * 
 * @note This function requires that the input string have the form 'yyyy.mm.dd-HH.MM.SS.fff'.
 * 
 * @ingroup utilities
 */
PDPUTILITIES_EXPORT std::string usbCameraDevicesInfo();

PDPUTILITIES_EXPORT bool usbCameraStreamOn();

PDPUTILITIES_EXPORT bool usbCameraStartVideoStream(
  int vendorId,
  int productId,
  enum uvc_frame_format format,
  int width,
  int height,
  int fps,
  bool autoExposureOn,
  void* displayFrameFunction,
  std::string& messages);

PDPUTILITIES_EXPORT void usbCameraStopVideoStream();

} // namespace pdp
